_MAJOR_VER = 0
_MINOR_VER = 7
_FIX_VER = 1
_BUILD_VER = 1
TR_BASE_VERSION = f'{_MAJOR_VER}.{_MINOR_VER}.{_FIX_VER}'
TR_FULL_VERSION = f'{_MAJOR_VER}.{_MINOR_VER}.{_FIX_VER}-{_BUILD_VER}'
__VERSION__ = TR_FULL_VERSION
